from __future__ import unicode_literals
import os
import re
import time
from ..utils.notebook.inflection import parameterize
from ..utils.notebook.data_uri import data_uri_from


class IPythonNotebook(object):
    _name = None
    _version = None
    thumbnail_file = None

    def __init__(self, filename, fileobj, *args, **kwargs):

        if isinstance(filename, bytes):
            filename = filename.decode('utf-8', errors='ignore')

        self.filename = filename
        self.thumbnail_file = kwargs.get('thumbnail_file', None)

    @property
    def basename(self):
        return os.path.basename(self.filename)

    @property
    def name(self):
        if self._name is None:
            return re.sub('\-ipynb$', '', parameterize(os.path.basename(self.filename)))
        return self._name

    @property
    def version(self):
        if self._version is None:
            self._version = time.strftime('%Y.%m.%d.%H%M')
        return self._version

    @property
    def thumbnail(self):
        if self.thumbnail_file is None:
            return None
        return data_uri_from(self.thumbnail_file)

    def get_package_data(self):
        if self.thumbnail_file is None:
            return {
                'name': self.name,
                'summary': 'IPython notebook'
            }
        else:
            return {
                'name': self.name,
                'summary': 'IPython notebook',
                'thumbnail': self.thumbnail
            }


def inspect_ipynb_package(filename, fileobj, *args, **kwargs):
    if 'parser_args' in kwargs:
        thumbnail_file = kwargs['parser_args'].thumbnail
        ipython_notebook = IPythonNotebook(filename, fileobj, thumbnail_file=thumbnail_file)
    else:
        ipython_notebook = IPythonNotebook(filename, fileobj)

    package_data = ipython_notebook.get_package_data()
    release_data = {
        'version': ipython_notebook.version,
        'description': ''
    }
    file_data = {
        'basename': ipython_notebook.basename,
        'attrs': {}
    }

    return package_data, release_data, file_data



'''
Remove an object from Anaconda Cloud

example::

    anaconda remove sean/meta/1.2.0/meta.tar.gz

'''
from binstar_client.utils import get_server_api, parse_specs, \
    bool_input
from argparse import RawTextHelpFormatter
from binstar_client import errors

import logging
log = logging.getLogger('binstar.remove')

def main(args):

    aserver_api = get_server_api(args.token, args.site, args.log_level)

    for spec in args.specs:
        try:
            if spec._basename:
                msg = 'Are you sure you want to remove file %s ?' % (spec,)
                if args.force or bool_input(msg, False):
                    aserver_api.remove_dist(spec.user, spec.package, spec.version, spec.basename)
                else:
                    log.warn('Not removing file %s' % (spec))
            elif spec._version:
                msg = 'Are you sure you want to remove the package release %s ? (and all files under it?)' % (spec,)
                if args.force or bool_input(msg, False):
                    aserver_api.remove_release(spec.user, spec.package, spec.version)
                else:
                    log.warn('Not removing release %s' % (spec))
            elif spec._package:
                msg = 'Are you sure you want to remove the package %s ? (and all data with it?)' % (spec,)
                if args.force or bool_input(msg, False):
                    aserver_api.remove_package(spec.user, spec.package)
                else:
                    log.warn('Not removing release %s' % (spec))
            else:
                log.error('Invalid package specification: %s', spec)

        except errors.NotFound:
            if args.force:
                log.warn('', exc_info=True)
                continue
            else:
                raise


def add_parser(subparsers):

    parser = subparsers.add_parser('remove',
                                      help='Remove an object from Anaconda Cloud. Must refer to the formal package name as it appears in the URL of the package. Also use anaconda show <USERNAME> to see list of pacakge names. Example: anaconda remove continuumio/empty-example-notebook',
                                      description=__doc__, formatter_class=RawTextHelpFormatter)

    parser.add_argument('specs', help='Package written as <user>[/<package>[/<version>[/<filename>]]]', type=parse_specs, nargs='+')
    parser.add_argument('-f', '--force', help='Do not prompt removal', action='store_true')



    parser.set_defaults(main=main)



'''
Anaconda Cloud command line manager
'''
from __future__ import print_function, unicode_literals

from argparse import ArgumentParser, RawDescriptionHelpFormatter
import logging
from os import makedirs
from os.path import join, exists, isfile

from binstar_client import __version__ as version
from binstar_client import commands as command_module
from binstar_client.commands.login import interactive_login
from binstar_client import errors
from binstar_client.utils import USER_LOGDIR
from binstar_client.utils.handlers import syslog_handler

from clyent import add_default_arguments, add_subparser_modules
from clyent.logs import setup_logging
import sys


logger = logging.getLogger('binstar')


def file_or_token(value):

    if isfile(value):
        with open(value) as fd:
            return fd.read().strip()
    if any(char in value for char in '/\\.'):
        # This chars will never be in a token value, but may be in a path
        # The error message will be handled by the parser
        raise ValueError()
    return value


def binstar_main(sub_command_module, args=None, exit=True, description=None, version=None, epilog=None):

    parser = ArgumentParser(description=description, epilog=epilog,
                            formatter_class=RawDescriptionHelpFormatter)

    add_default_arguments(parser, version)
    bgroup = parser.add_argument_group('anaconda-client options')
    bgroup.add_argument('-t', '--token', type=file_or_token,
                        help="Authentication token to use. "
                             "May be a token or a path to a file containing a token")
    bgroup.add_argument('-s', '--site',
                        help='select the anaconda-client site to use', default=None)

    add_subparser_modules(parser, sub_command_module, 'conda_server.subcommand')

    if not exists(USER_LOGDIR): makedirs(USER_LOGDIR)
    logfile = join(USER_LOGDIR, 'cli.log')

    args = parser.parse_args(args)

    setup_logging(logger, args.log_level, use_color=args.color,
                  logfile=logfile, show_tb=args.show_traceback)

    try:
        try:
            if not hasattr(args, 'main'):
                parser.error("A sub command must be given. "
                             "To show all available sub commands, run:\n\n\t anaconda -h\n")
            return args.main(args)
        except errors.Unauthorized:
            if not sys.stdin.isatty() or args.token:
                # Don't try the interactive login
                # Just exit
                raise

            logger.info('The action you are performing requires authentication, '
                        'please sign in:')
            interactive_login(args)
            return args.main(args)

    except errors.ShowHelp:
        args.sub_parser.print_help()
        if exit:
            raise SystemExit(1)
        else:
            return 1


def main(args=None, exit=True):
    binstar_main(command_module, args, exit,
                 description=__doc__, version=version)

if __name__ == '__main__':
    main()



'''
Created on Aug 8, 2013

@author: sean
'''
from __future__ import unicode_literals
from dateutil.parser import parse as parse_date
import logging
log = logging.getLogger('binstar.pprint')

def pprint_orgs(orgs):
    log.info('Organizations:')
    for org in orgs:
        log.info('   + %(login)25s' % org)

fmt_access = '     %(full_name)-25s | %(latest_version)8s | %(access)-12s | %(package_types)-15s | %(conda_platforms)-15s'
fmt_no_access = '     %(full_name)-25s | %(latest_version)8s | %(package_types)-15s | %(conda_platforms)-15s'

def pprint_package_header(access=True, revisions=False):
    package_header = {'full_name':'Name',
                      'access': 'Access', 'package_types':'Package Types',
                      'latest_version': 'Version', 'conda_platforms': 'Platforms',
                      'revision':'Rev'}
    fmt = fmt_access if access else fmt_no_access
    if revisions:
        fmt = '%(revision)-6s | ' + fmt

    log.info(fmt % package_header)

def pprint_package(package, access=True, full_name=True, revision=False):
    package = package.copy()

    if 'conda_platforms' in package:
        package['conda_platforms'] = ', '.join(package['conda_platforms'])
    if not full_name:
        package['full_name'] = package['name']
    package['access'] = 'published' if package.get('published') else 'public' if package['public'] else 'private'
    if package.get('package_types'):
        package['package_types'] = ', '.join(package['package_types'])
    fmt = fmt_access if access else fmt_no_access

    if revision:
        fmt = '%(revision)-6s | ' + fmt

    log.info(fmt % package)
    if package.get('summary'):
        log.info(' ' * 34 + '        : %s' % package.get('summary'))

def pprint_packages(packages, access=True, full_name=True, revisions=False):
    if packages:
        log.info('Packages:')
    else:
        log.info('No packages found')

    fmt = fmt_access if access else fmt_no_access
    if revisions:
        fmt = '%(revision)-6s | ' + fmt
    pprint_package_header(access, revisions=revisions)

    package_header = {'full_name':'-' * 25, 'access': '-' * 12,
                      'latest_version': '-' * 6, 'conda_platforms': '-' * 15,
                      'package_types':'-' * 15,
                      'revision': '-' * 6}

    log.info(fmt % package_header)

    key = lambda pkg: pkg['full_name'] if full_name else pkg['name']

    for package in sorted(packages, key=key):
        pprint_package(package, access, full_name, revision=revisions)

def pprint_user(user):
    user = user.copy()
    log.info('Username: %s', user.pop('login'))
    log.info('Member since: %s', parse_date(user.pop('created_at')).ctime())

    for key_value in user.items():
        log.info('  +%s: %s' % key_value)

def pprint_collections(collections):
    if collections:
        log.info('Collections:')
    for collection in collections:
        collection['permission'] = 'public' if collection['public'] else 'private'
        log.info('   + %(name)25s: [%(permission)s] %(description)s' % collection)




'''
Usage:
    anaconda download notebook
    anaconda download user/notebook
'''

from __future__ import unicode_literals

import argparse
import logging

from binstar_client import errors
from binstar_client.utils import get_server_api
from binstar_client.utils.notebook import Downloader, parse, has_environment

log = logging.getLogger("binstar.download")


def add_parser(subparsers):
    description = 'Download notebooks from Anaconda Cloud'
    parser = subparsers.add_parser(
        'download',
        formatter_class=argparse.RawDescriptionHelpFormatter,
        help=description,
        description=description,
        epilog=__doc__
    )

    parser.add_argument(
        'handle',
        help="user/notebook",
        action='store'
    )

    parser.add_argument(
        '-f', '--force',
        help='Overwrite',
        action='store_true'
    )

    parser.add_argument(
        '-o', '--output',
        help='Download as',
        default='.'
    )

    parser.set_defaults(main=main)


def main(args):
    aserver_api = get_server_api(args.token, args.site, args.log_level)
    username, notebook = parse(args.handle)
    username = username or aserver_api.user()['login']
    downloader = Downloader(aserver_api, username, notebook)
    try:
        download_info = downloader(output=args.output, force=args.force)
        log.info("{} has been downloaded as {}.".format(args.handle, download_info[0]))
        if has_environment(download_info[0]):
            log.info("{} has an environment embedded.".format(download_info[0]))
            log.info("Run:")
            log.info("    conda env create {}".format(download_info[0]))
            log.info("To install the environment in your system")
    except (errors.DestionationPathExists, errors.NotFound, errors.BinstarError, OSError) as err:
        log.info(err.msg)



'''
Created on Feb 18, 2014

@author: sean
'''
from __future__ import unicode_literals
from binstar_client.scripts.cli import main
from binstar_client.tests.fixture import CLITestCase
from binstar_client.tests.urlmock import urlpatch
import base64
import json
import unittest

@unittest.skip("Need to change this to binsar package --create")
class Test(CLITestCase):

    @urlpatch
    def test_register_public(self, registry):

        r1 = registry.register(method='GET', path='/user', content='{"login": "eggs"}')
        r2 = registry.register(method='GET', path='/package/eggs/foo', status=404)
        r3 = registry.register(method='POST', path='/package/eggs/foo', status=200, content='{"login": "eggs"}')

        main(['--show-traceback', 'register', self.data_dir('foo-0.1-0.tar.bz2')], False)

        r1.assertCalled()
        r2.assertCalled()
        r3.assertCalled()

        data = json.loads(r3.req.body)
        self.assertTrue(data['public'])

    @urlpatch
    def test_register_private(self, registry):

        r1 = registry.register(method='GET', path='/user', content='{"login": "eggs"}')
        r2 = registry.register(method='GET', path='/package/eggs/foo', status=404)
        r3 = registry.register(method='POST', path='/package/eggs/foo', status=200, content='{"login": "eggs"}')

        main(['--show-traceback', 'register', '--private', self.data_dir('foo-0.1-0.tar.bz2')], False)

        r1.assertCalled()
        r2.assertCalled()
        r3.assertCalled()

        data = json.loads(r3.req.body)
        self.assertFalse(data['public'])



if __name__ == '__main__':
    unittest.main()



import logging
import traceback
import json
import os
import socket

class JSONFormatter(object):

    def __init__(self, *args, **extra_tags):
        self.dumps = extra_tags.pop('dumps', lambda obj: json.dumps(obj, default=lambda obj: str(obj)))

        object.__init__(self, *args)
        self.extra_tags = extra_tags

    def format(self, record):

        if isinstance(record.msg, dict):
            data = record.msg
        elif isinstance(record.msg, (list, tuple)):
            data = {'items': record.msg}
        else:
            data = {'msg':record.msg}

        kwargs = self.extra_tags.copy()

        data.update(logLevel=record.levelname,
                    logModule=record.module,
                    logName=record.name,
                    pid=os.getpid(),
                    **kwargs)

        if record.exc_info:
            etype, value, tb = record.exc_info
            tb = '\n'.join(traceback.format_exception(etype, value, tb))
            data['exception'] = True
            data['traceback'] = tb

        msg = self.dumps(data)
        return msg

class JSONSysLogFormatter(JSONFormatter):
    def __init__(self, appName, *args, **extra_tags):
        self.appName = appName
        JSONFormatter.__init__(self, *args, **extra_tags)

    def format(self, record):
        msg = JSONFormatter.format(self, record)
        return '%s %s' % (self.appName, msg)


def syslog_handler(app_name='binstar-client'):

    address = None
    if os.path.exists('/dev/log'):
        address = '/dev/log'
    elif os.path.exists('/var/run/syslog'):
        address = '/var/run/syslog'
    else:
        address = ('localhost', 514)
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        try:
            s.connect(address)
        except:
            return logging.NullHandler()

    hdlr = logging.handlers.SysLogHandler(address=address)
    hdlr.setLevel(logging.INFO)
    hdlr.setFormatter(JSONSysLogFormatter(app_name))
    return hdlr



try:
    import nbformat
except ImportError:
    nbformat = None

try:
    from urlparse import urlparse
except ImportError:
    from urllib.parse import urlparse
from .uploader import *
from .downloader import *
from ...errors import BinstarError


def parse(handle):
    """
    >>> parse("user/notebook")
    ('user', 'notebook')
    >>> parse("notebook")
    (None, 'notebook')

    :param handle: String
    :return: username, notebooks
    """

    components = handle.split('/', 1)
    if len(components) == 1:
        return None, components[0]
    elif len(components) == 2:
        return components[0], components[1]
    else:
        raise BinstarError("{} can't be parsed".format(handle))


def notebook_url(upload_info):
    parsed = urlparse(upload_info['url'])
    if parsed.netloc == 'anaconda.org':
        url = "{}://notebooks.{}{}".format(parsed.scheme, parsed.netloc, parsed.path)
    else:
        url = "{}://{}/notebooks{}".format(parsed.scheme, parsed.netloc, parsed.path)
    return url


def has_environment(nb_file):
    try:
        with open(nb_file) as fb:
            data = fb.read()
        nb = nbformat.reader.reads(data)
        return 'environment' in nb['metadata']
    except (AttributeError, KeyError):
        return False
    except (IOError, nbformat.reader.NotJSONError):
        raise BinstarError("Unable to open {}.".format(nb_file))



import unittest
from os.path import join, dirname
from binstar_client.utils.notebook.data_uri import DataURIConverter


class DataURIConverterTestCase(unittest.TestCase):
    def data_dir(self, filename):
        test_data = join(dirname(__file__), 'data')
        return join(test_data, filename)

    def test_local_image(self):
        location = self.data_dir('bokeh-logo.png')
        output = DataURIConverter(location)()
        self.assertEqual(output[0:5], "iVBOR")

    def test_remote_image(self):
        location = 'http://continuum.io/media/img/anaconda_server_logo.png'
        output = DataURIConverter(location)()
        self.assertEqual(output[0:5], "iVBOR")

    def test_file_not_found(self):
        location = self.data_dir('no-exists.png')
        with self.assertRaises(IOError):
            DataURIConverter(location)()

    def test_is_python_3(self):
        output = DataURIConverter('')
        self.assertIsInstance(output.is_py3(), bool)

    def test_is_url(self):
        location = 'http://docs.continuum.io/_static/img/continuum_analytics_logo.png'
        self.assertTrue(DataURIConverter(location).is_url())

        location = self.data_dir('bokeh-logo.png')
        self.assertNotEqual(DataURIConverter(location).is_url(), True)



'''
Search Anaconda Cloud for packages
'''
from binstar_client.utils import get_server_api
from binstar_client.utils.pprint import pprint_packages
import logging
log = logging.getLogger('binstar.search')

def search(args):

    aserver_api = get_server_api(args.token, args.site, args.log_level)

    log.info("Run 'anaconda show <USER/PACKAGE>' to get more details:")
    packages = aserver_api.search(args.name, package_type=args.package_type)
    pprint_packages(packages, access=False)
    log.info("Found %i packages" % len(packages))


def add_parser(subparsers):
    parser1 = subparsers.add_parser('search',
                                      help='Search Anaconda Cloud',
                                      description='Search Anaconda Cloud',
                                      epilog=__doc__)
    parser1.add_argument('name', nargs=1, help='Search string')
    parser1.add_argument('-t', '--package-type', choices=['conda', 'pypi'],
                         help='only search for packages of this type')
    parser1.set_defaults(main=search)



import os

from binstar_client.utils import conda


def test_conda_root():
    from binstar_client.utils.conda import get_conda_root

    assert get_conda_root() is not None


def test_conda_root_outside_root_environment(monkeypatch):
    called_mock = { 'called' : False }
    def mock_import_conda_root():
        called_mock['called'] = True
        raise ImportError("did not import it")

    monkeypatch.setattr('binstar_client.utils.conda._import_conda_root', mock_import_conda_root)

    from binstar_client.utils.conda import get_conda_root

    assert get_conda_root() is not None

    assert called_mock['called']


def test_conda_root_from_conda_info(monkeypatch):
    from binstar_client.utils.conda import _conda_root_from_conda_info

    conda_root = _conda_root_from_conda_info()
    assert conda_root is not None
    assert os.path.isdir(conda_root)



from os.path import basename, dirname, join
import json
import sys
import subprocess

WINDOWS = sys.platform.startswith('win')
CONDA_PREFIX = sys.prefix
BIN_DIR = 'Scripts' if WINDOWS else 'bin'
CONDA_EXE = join(CONDA_PREFIX, BIN_DIR,
                 'conda.exe' if WINDOWS else 'conda')


# this function is broken out for monkeypatch by unit tests,
# so we can test the ImportError handling
def _import_conda_root():
    import conda.config
    return conda.config.root_dir


def _conda_root_from_conda_info():
    try:
        output = subprocess.check_output([CONDA_EXE, 'info', '--json']).decode("utf-8")
        conda_info = json.loads(output)
        return conda_info['root_prefix']
    except (ValueError, KeyError, subprocess.CalledProcessError):
        return None


def get_conda_root():
    """Get the PREFIX of the conda installation.

    Returns:
        str: the ROOT_PREFIX of the conda installation
    """
    try:
        # Fast-path
        # We're in the root environment
        conda_root = _import_conda_root()
    except ImportError:
        # We're not in the root environment.
        envs_dir = dirname(CONDA_PREFIX)
        if basename(envs_dir) == 'envs':
            # We're in a named environment: `conda create -n <name>`
            conda_root = dirname(envs_dir)
        else:
            # We're in an isolated environment: `conda create -p <path>`
            # The only way we can find out is by calling conda.
            conda_root = _conda_root_from_conda_info()

    return conda_root

CONDA_ROOT = get_conda_root()
